@@ -14,30 +14,24 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with Cockpit; If not, see <http://www.gnu.org/licenses/>.
+ * along with Cockpit; If not, see <https://www.gnu.org/licenses/>.
  */
 
 import cockpit from "cockpit";
 import React from "react";
 
+import { Flex } from "@patternfly/react-core/dist/esm/layouts/Flex/index.js";
+import { Spinner } from "@patternfly/react-core/dist/esm/components/Spinner/index.js";
 import { WarningTriangleIcon } from "@patternfly/react-icons";
 
 import { show_modal_dialog } from "cockpit-components-dialog.jsx";
-import * as PK from "packagekit.js";
+import { getPackageManager, InstallProgressType } from "packagemanager";
 
 import "cockpit-components-install-dialog.css";
+import { fmt_to_fragments } from "utils";
 
 const _ = cockpit.gettext;
 
-// TODO - generalize this to arbitrary number of arguments (when needed)
-function format_to_fragments(fmt, arg) {
-    const index = fmt.indexOf("$0");
-    if (index >= 0)
-        return <>{fmt.slice(0, index)}{arg}{fmt.slice(index + 2)}</>;
-    else
-        return fmt;
-}
-
 /* Calling install_dialog will open a dialog that lets the user
  * install the given package.
  *
@@ -52,7 +46,7 @@ function format_to_fragments(fmt, arg) {
  * (If you do anyway, the resulting D-Bus errors will be shown to the user.)
  */
 
-export function install_dialog(pkg, options) {
+export async function install_dialog(pkg, options) {
     let data = null;
     let error_message = null;
     let progress_message = null;
@@ -64,6 +58,7 @@ export function install_dialog(pkg, options) {
 
     options = options || { };
 
+    const package_manager = await getPackageManager();
     const prom = new Promise((resolve, reject) => { done = f => { if (f) resolve(); else reject(); } });
 
     let dialog = null;
@@ -92,15 +87,15 @@ export function install_dialog(pkg, options) {
 
         if (progress_message)
             footer_message = (
-                <>
+                <Flex spaceItems={{ default: 'spaceItemsSm' }} alignItems={{ default: 'alignItemsCenter' }}>
                     <span>{ progress_message }</span>
-                    <div className="spinner spinner-sm" />
-                </>
+                    <Spinner size="sm" />
+                </Flex>
             );
-        else if (data && data.download_size) {
+        else if (data?.download_size) {
             footer_message = (
                 <div>
-                    { format_to_fragments(_("Total size: $0"), <strong>{cockpit.format_bytes(data.download_size)}</strong>) }
+                    { fmt_to_fragments(_("Total size: $0"), <strong>{cockpit.format_bytes(data.download_size)}</strong>) }
                 </div>
             );
         }
@@ -110,7 +105,7 @@ export function install_dialog(pkg, options) {
             title: options.title || _("Install software"),
             body: (
                 <div className="scroll">
-                    <p>{ format_to_fragments(options.text || _("$0 will be installed."), missing_name) }</p>
+                    <p>{ fmt_to_fragments(options.text || _("$0 will be installed."), missing_name) }</p>
                     { remove_details }
                     { extra_details }
                 </div>
@@ -140,19 +135,19 @@ export function install_dialog(pkg, options) {
     }
 
     function check_missing() {
-        PK.check_missing_packages(pkg,
-                                  p => {
-                                      cancel = p.cancel;
-                                      let pm = null;
-                                      if (p.waiting)
-                                          pm = _("Waiting for other software management operations to finish");
-                                      else
-                                          pm = _("Checking installed software");
-                                      if (pm != progress_message) {
-                                          progress_message = pm;
-                                          update();
-                                      }
-                                  })
+        package_manager.check_missing_packages(pkg,
+                                               p => {
+                                                   cancel = p.cancel;
+                                                   let pm = null;
+                                                   if (p.waiting)
+                                                       pm = _("Waiting for other software management operations to finish");
+                                                   else
+                                                       pm = _("Checking installed software");
+                                                   if (pm != progress_message) {
+                                                       progress_message = pm;
+                                                       update();
+                                                   }
+                                               })
                 .then(d => {
                     if (d.unavailable_names.length > 0)
                         error_message = cockpit.format(_("$0 is not available from any repository."),
@@ -171,36 +166,24 @@ export function install_dialog(pkg, options) {
                 });
     }
 
-    function install_missing() {
-        // We need to return a Cockpit flavoured promise since we want
-        // to use progress notifications.
-        const dfd = cockpit.defer();
-
-        PK.install_missing_packages(data,
-                                    p => {
-                                        let text = null;
-                                        if (p.waiting) {
-                                            text = _("Waiting for other software management operations to finish");
-                                        } else if (p.package) {
-                                            let fmt;
-                                            if (p.info == PK.Enum.INFO_DOWNLOADING)
-                                                fmt = _("Downloading $0");
-                                            else if (p.info == PK.Enum.INFO_REMOVING)
-                                                fmt = _("Removing $0");
-                                            else
-                                                fmt = _("Installing $0");
-                                            text = format_to_fragments(fmt, <strong>{p.package}</strong>);
-                                        }
-                                        dfd.notify(text, p.cancel);
-                                    })
-                .then(() => {
-                    dfd.resolve();
-                })
-                .catch(error => {
-                    dfd.reject(error);
-                });
-
-        return dfd.promise;
+    function install_missing(progress_cb) {
+        return package_manager.install_missing_packages(data,
+                                                        p => {
+                                                            let text = null;
+                                                            if (p.waiting) {
+                                                                text = _("Waiting for other software management operations to finish");
+                                                            } else if (p.package) {
+                                                                let fmt;
+                                                                if (p.info == InstallProgressType.DOWNLOADING)
+                                                                    fmt = _("Downloading $0");
+                                                                else if (p.info == InstallProgressType.REMOVING)
+                                                                    fmt = _("Removing $0");
+                                                                else
+                                                                    fmt = _("Installing $0");
+                                                                text = fmt_to_fragments(fmt, <strong>{p.package}</strong>);
+                                                            }
+                                                            progress_cb(text, p.cancel);
+                                                        });
     }
 
     update();
