@@ -14,10 +14,11 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with Cockpit; If not, see <http://www.gnu.org/licenses/>.
+ * along with Cockpit; If not, see <https://www.gnu.org/licenses/>.
  */
 
 import cockpit from "cockpit";
+import * as ipaddr from "ipaddr.js";
 
 const _ = cockpit.gettext;
 
@@ -55,6 +56,35 @@ export function ip_metric_from_text(text) {
     throw cockpit.format(_("Invalid metric $0"), text);
 }
 
+export function ip_network_address(address, prefix_len) {
+    const addrCIDR = address.toString() + "/" + prefix_len;
+    const ip_obj = address.kind() === "ipv4" ? ipaddr.IPv4 : ipaddr.IPv6;
+
+    try {
+        return ip_obj.networkAddressFromCIDR(addrCIDR).toString();
+    } catch (_e) {
+        return null;
+    }
+}
+
+export function ip_first_usable_address(address, prefix) {
+    try {
+        const addr_cidr = address.toString() + "/" + prefix;
+
+        if (address.kind() === "ipv4") {
+            const netAddr = ipaddr.IPv4.networkAddressFromCIDR(addr_cidr);
+            netAddr.octets[3] += (prefix < 31) ? 1 : 0;
+            return netAddr.toString();
+        } else {
+            const netAddr = ipaddr.IPv6.networkAddressFromCIDR(addr_cidr);
+            netAddr.parts[7] += (prefix < 127) ? 1 : 0;
+            return netAddr.toString();
+        }
+    } catch (_e) {
+        return null;
+    }
+}
+
 function toDec(n) {
     return n.toString(10);
 }
@@ -77,6 +107,20 @@ function bytes_from_nm32(num) {
     return bytes;
 }
 
+export function validate_ipv4(address) {
+    // explicitly require all 4 octets
+    // NM does not support any IPv4 short format
+    return ipaddr.IPv4.isValidFourPartDecimal(address);
+}
+
+export function validate_ipv6(address) {
+    return ipaddr.IPv6.isValid(address);
+}
+
+export function validate_ip(address) {
+    return validate_ipv4(address) || validate_ipv6(address);
+}
+
 export function ip4_to_text(num, zero_is_empty) {
     if (num === 0 && zero_is_empty)
         return "";
@@ -125,36 +169,57 @@ export function ip4_from_text(text, empty_is_zero) {
     return num;
 }
 
-const text_to_prefix_bits = {
-    255: 8, 254: 7, 252: 6, 248: 5, 240: 4, 224: 3, 192: 2, 128: 1, 0: 0
-};
-
-export function ip4_prefix_from_text(text) {
-    function invalid() {
-        throw cockpit.format(_("Invalid prefix or netmask $0"), text);
+export function ip4_prefix_from_text(prefix_mask) {
+    const trimmed_mask = prefix_mask.trim();
+    if (/^[0-9]+$/.test(trimmed_mask)) {
+        return parseInt(prefix_mask, 10);
     }
 
-    if (/^[0-9]+$/.test(text.trim()))
-        return parseInt(text, 10);
-    const parts = text.split('.');
-    if (parts.length != 4)
-        invalid();
-    let prefix = 0;
-    let i;
-    for (i = 0; i < 4; i++) {
-        const p = text_to_prefix_bits[parts[i].trim()];
-        if (p !== undefined) {
-            prefix += p;
-            if (p < 8)
-                break;
-        } else
-            invalid();
+    // make sure that mask has 4 octets format
+    if (validate_ipv4(trimmed_mask)) {
+        const prefix = ipaddr.IPv4.parse(trimmed_mask).prefixLengthFromSubnetMask();
+        if (prefix !== null) {
+            return prefix;
+        }
     }
-    for (i += 1; i < 4; i++) {
-        if (/^0+$/.test(parts[i].trim()) === false)
-            invalid();
+
+    throw cockpit.format(_("Invalid prefix or netmask $0"), prefix_mask);
+}
+
+// Shorten IPv6 address according to RFC 5952
+// https://datatracker.ietf.org/doc/html/rfc5952#section-4
+//
+// NetworkManager already handles dropping of leadin zeros within a single 16 bit field
+// but does not replace the longest consecutive zeros fields with "::".
+function ip6_shorten(ip6_addr) {
+    function find_longest_zero(match) {
+        let idx = -1;
+        let length = 0;
+
+        match.forEach((item, i) => {
+            const count_zero = item[0].replaceAll(':', '').length;
+            if (count_zero > length) {
+                idx = i;
+                length = count_zero;
+            }
+        });
+
+        return idx;
     }
-    return prefix;
+
+    const REGEX_MATCH_CONSECUTIVE_ZEROS = /\b:?(?:0:?){2,}/g;
+    const match = [...ip6_addr.matchAll(REGEX_MATCH_CONSECUTIVE_ZEROS)];
+
+    // nothing to shorten
+    if (match.length === 0) {
+        return ip6_addr;
+    }
+
+    const longest_idx = find_longest_zero(match);
+    // replace first (leftmost) match
+    const short_addr = ip6_addr.replace(match[longest_idx], "::");
+
+    return short_addr;
 }
 
 export function ip6_to_text(data, zero_is_empty) {
@@ -165,7 +230,7 @@ export function ip6_to_text(data, zero_is_empty) {
     const result = parts.join(':');
     if (result == "0:0:0:0:0:0:0:0" && zero_is_empty)
         return "";
-    return result;
+    return ip6_shorten(result);
 }
 
 export function ip6_from_text(text, empty_is_zero) {
