@@ -76,10 +76,12 @@ import * as PK from "packagekit.js";
 import * as python from "python.js";
 import * as timeformat from "timeformat";
 
-import { debug } from './utils';
+import { debug, watchRedHatSubscription } from './utils';
 import callTracerScript from './callTracer.py';
 
 import "./updates.scss";
+import { Truncate } from '@patternfly/react-core/dist/esm/components/Truncate/index.js';
+import { Severity } from '_internal/packagemanager-abstract';
 
 const _ = cockpit.gettext;
 
@@ -87,7 +89,6 @@ const _ = cockpit.gettext;
 const STATE_HEADINGS = {};
 const PK_STATUS_STRINGS = {};
 const PK_STATUS_LOG_STRINGS = {};
-const packageSummaries = {};
 
 const UPDATES = {
     ALL: 0,
@@ -117,23 +118,8 @@ function init() {
     PK_STATUS_LOG_STRINGS[PK.Enum.STATUS_SIGCHECK] = _("Verified");
 }
 
-// parse CVEs from an arbitrary text (changelog) and return URL array
-function parseCVEs(text) {
-    if (!text)
-        return [];
-
-    const cves = text.match(/CVE-\d{4}-\d+/g);
-    if (!cves)
-        return [];
-    return cves.map(n => "https://www.cve.org/CVERecord?id=" + n);
-}
-
 function deduplicate(list) {
-    const d = { };
-    list.forEach(i => { if (i) d[i] = true; });
-    const result = Object.keys(d);
-    result.sort();
-    return result;
+    return [...new Set(list)].sort();
 }
 
 // Insert comma strings in between elements of the list. Unlike list.join(",")
@@ -161,20 +147,13 @@ function cleanupChangelogLine(text) {
 
 // Replace cockpit-wsinstance-https@[long_id] with a shorter string
 function shortenCockpitWsInstance(list) {
-    list = [...list];
-
-    list.forEach((item, idx) => {
-        if (item.startsWith("cockpit-wsinstance-https"))
-            list[idx] = "cockpit-wsinstance-https@.";
-    });
-
-    return list;
+    return list.map(item => item.startsWith('cockpit-wsinstance-https') ? 'cockpit-wsinstance-https@.' : item);
 }
 
 function count_security_updates(updates) {
     let num_security = 0;
-    for (const u in updates)
-        if (updates[u].severity === PK.Enum.INFO_SECURITY)
+    for (const u of updates)
+        if (u.severity === Severity.CRITICAL)
             ++num_security;
     return num_security;
 }
@@ -185,25 +164,25 @@ function isKpatchPackage(name) {
 
 function count_kpatch_updates(updates) {
     let num_kpatches = 0;
-    for (const u in updates)
-        if (isKpatchPackage(updates[u].name))
+    for (const u of updates)
+        if (isKpatchPackage(u.name))
             ++num_kpatches;
     return num_kpatches;
 }
 
 function find_highest_severity(updates) {
-    let max = PK.Enum.INFO_LOW;
-    for (const u in updates)
-        if (updates[u].severity > max)
-            max = updates[u].severity;
+    let max = Severity.LOW;
+    for (const u of updates)
+        if (u.severity > max)
+            max = u.severity;
     return max;
 }
 
 /**
  * Get appropriate icon for an update severity
  *
- * info: An Enum.INFO_* level
- * secSeverity: If given, further classification of the severity of Enum.INFO_SECURITY from the vendor_urls;
+ * info: An Severity level
+ * secSeverity: If given, further classification of the severity of Severity.CRITICAL from the vendor_urls;
  *              e. g. "critical", see https://access.redhat.com/security/updates/classification
  * Returns: Icon JSX object
  *
@@ -212,18 +191,18 @@ function getSeverityIcon(info, secSeverity) {
     let classes = "severity-icon";
     if (secSeverity)
         classes += " severity-" + secSeverity;
-    if (info == PK.Enum.INFO_SECURITY)
+    if (info == Severity.CRITICAL)
         return <SecurityIcon aria-label={ secSeverity || _("security") } className={classes} />;
-    else if (info >= PK.Enum.INFO_NORMAL)
+    else if (info >= Severity.IMPORTANT)
         return <BugIcon className={classes} aria-label={ _("bug fix") } />;
     else
         return <EnhancementIcon className={classes} aria-label={ _("enhancement") } />;
 }
 
 function getPageStatusSeverityIcon(severity) {
-    if (severity == PK.Enum.INFO_SECURITY)
+    if (severity == Severity.CRITICAL)
         return "security";
-    else if (severity >= PK.Enum.INFO_NORMAL)
+    else if (severity >= Severity.IMPORTANT)
         return "bug";
     else
         return "enhancement";
@@ -306,7 +285,7 @@ function updateItem(remarkable, info, pkgNames, key) {
     const secSeverity = secSeverityURL ? secSeverityURL.slice(secSeverityURL.indexOf("#") + 1) : null;
     const icon = getSeverityIcon(info.severity, secSeverity);
     let type;
-    if (info.severity === PK.Enum.INFO_SECURITY) {
+    if (info.severity === Severity.CRITICAL) {
         if (secSeverityURL)
             secSeverityURL = <a rel="noopener noreferrer" target="_blank" href={secSeverityURL}>{secSeverity}</a>;
         type = (
@@ -318,7 +297,7 @@ function updateItem(remarkable, info, pkgNames, key) {
             </Tooltip>
         );
     } else {
-        const tip = (info.severity >= PK.Enum.INFO_NORMAL) ? _("bug fix") : _("enhancement");
+        const tip = (info.severity >= Severity.IMPORTANT) ? _("bug fix") : _("enhancement");
         type = (
             <Tooltip id="tip-severity" content={tip}>
                 <span>
@@ -330,7 +309,7 @@ function updateItem(remarkable, info, pkgNames, key) {
     }
 
     const pkgList = pkgNames.map((n, index) => (
-        <Tooltip key={n.name + n.arch} id="tip-summary" content={packageSummaries[n.name] + " (" + n.arch + ")"}>
+        <Tooltip key={n.name + n.arch} id="tip-summary" content={n.summary + " (" + n.arch + ")"}>
             <span>{n.name + (index !== (pkgNames.length - 1) ? ", " : "")}</span>
         </Tooltip>)
     );
@@ -404,7 +383,7 @@ function updateItem(remarkable, info, pkgNames, key) {
         ],
         props: {
             key,
-            className: info.severity === PK.Enum.INFO_SECURITY ? ["error"] : [],
+            className: info.severity === Severity.CRITICAL ? ["error"] : [],
         },
         hasPadding: true,
         expandedContent,
@@ -413,34 +392,33 @@ function updateItem(remarkable, info, pkgNames, key) {
 
 const UpdatesList = ({ updates }) => {
     const remarkable = customRemarkable();
-    const update_ids = [];
+    const combined_updates = [];
 
     // PackageKit doesn"t expose source package names, so group packages with the same version and changelog
     // create a reverse version+changes → [id] map on iteration
     const sameUpdate = {};
     const packageNames = {};
-    Object.keys(updates).forEach(id => {
-        const u = updates[id];
+    for (const u of updates) {
         // did we already see the same version and description? then merge
         const hash = u.version + u.description;
         const seenId = sameUpdate[hash];
         if (seenId) {
-            packageNames[seenId].push({ name: u.name, arch: u.arch });
+            packageNames[seenId].push({ name: u.name, arch: u.arch, summary: u.summary });
         } else {
             // this is a new update
-            sameUpdate[hash] = id;
-            packageNames[id] = [{ name: u.name, arch: u.arch }];
-            update_ids.push(id);
+            sameUpdate[hash] = u.id;
+            packageNames[u.id] = [{ name: u.name, arch: u.arch, summary: u.summary }];
+            combined_updates.push(u);
         }
-    });
+    }
 
     // sort security first
-    update_ids.sort((a, b) => {
-        if (updates[a].severity === PK.Enum.INFO_SECURITY && updates[b].severity !== PK.Enum.INFO_SECURITY)
+    combined_updates.sort((a, b) => {
+        if (a.severity === Severity.CRITICAL && b.severity !== Severity.CRITICAL)
             return -1;
-        if (updates[a].severity !== PK.Enum.INFO_SECURITY && updates[b].severity === PK.Enum.INFO_SECURITY)
+        if (a.severity !== Severity.CRITICAL && b.severity === Severity.CRITICAL)
             return 1;
-        return a.localeCompare(b);
+        return a.name.localeCompare(b.name);
     });
 
     return (
@@ -452,7 +430,7 @@ const UpdatesList = ({ updates }) => {
                     { title: _("Severity"), props: { width: 15 } },
                     { title: _("Details"), props: { width: 30 } },
                 ]}
-                rows={update_ids.map(id => updateItem(remarkable, updates[id], packageNames[id].sort((a, b) => a.name > b.name), id))} />
+                rows={combined_updates.map(update => updateItem(remarkable, update, packageNames[update.id].sort((a, b) => a.name > b.name), update.id))} />
     );
 };
 
@@ -597,12 +575,14 @@ const ApplyUpdates = ({ transactionProps, actions, onCancel, rebootAfter, setReb
         <div className="progress-main-view">
             <Grid hasGutter>
                 <GridItem span={12}>
-                    <div className="progress-description">
-                        <Spinner size="md" />
-                        <strong>{ PK_STATUS_STRINGS[lastAction?.status] || PK_STATUS_STRINGS[PK.Enum.STATUS_UPDATE] }</strong>
-                        &nbsp;{curPackage}
+                    <div className="progress-description pf-v6-u-display-flex">
+                        <Spinner size="md" isInline />
+                        <strong>{PK_STATUS_STRINGS[lastAction?.status] || PK_STATUS_STRINGS[PK.Enum.STATUS_UPDATE]}</strong>
+                        &nbsp;
+                        <Truncate content={curPackage} />
                     </div>
                     <Progress title={remain}
+                              aria-label={remain ? _("Time remaining: ") : _("Update progress")}
                               value={percentage}
                               size={ProgressSize.sm}
                               className="pf-v6-u-mb-xs" />
@@ -689,7 +669,6 @@ const UpdateSuccess = ({ onIgnore, openServiceRestartDialog, openRebootDialog, r
             <EmptyStatePanel icon={RebootingIcon}
                              title={ _("Update was successful") }
                              headingLevel="h5"
-                             titleSize="4xl"
                              paragraph={paragraph}
                              secondary={actions} />
             <div className="flow-list-blank-slate">
@@ -754,7 +733,6 @@ const UpdateSuccess = ({ onIgnore, openServiceRestartDialog, openRebootDialog, r
     return (<>
         <EmptyStatePanel title={ _("Update was successful") }
             headingLevel="h5"
-            titleSize="4xl"
             secondary={
                 <>
                     { entries.length > 0 && <ListingTable aria-label={_("Update Success Table")}
@@ -780,7 +758,7 @@ const UpdateSuccess = ({ onIgnore, openServiceRestartDialog, openRebootDialog, r
 };
 
 const UpdatesStatus = ({ updates, highestSeverity, timeSinceRefresh, restartPackages, onValueChanged }) => {
-    const numUpdates = Object.keys(updates).length;
+    const numUpdates = updates.length;
     const numSecurity = count_security_updates(updates);
     const numRestartServices = restartPackages.daemons.length;
     const numManualSoftware = restartPackages.manual.length;
@@ -913,7 +891,7 @@ class CardsPage extends React.Component {
             className: settingsContent !== null ? "ct-card-info" : "",
             title: _("Status"),
             actions: (<Tooltip content={_("Check for updates")}>
-                <Button variant="secondary" onClick={this.props.handleRefresh}><RedoIcon /></Button>
+                <Button icon={<RedoIcon />} variant="secondary" onClick={this.props.handleRefresh} />
             </Tooltip>),
             body: statusContent,
         });
@@ -987,7 +965,7 @@ class OsUpdates extends React.Component {
         this.state = {
             state: "loading",
             errorMessages: [],
-            updates: {},
+            updates: [],
             timeSinceRefresh: null,
             loadPercent: null,
             cockpitUpdate: false,
@@ -1073,9 +1051,9 @@ class OsUpdates extends React.Component {
                     debug("tracer succeeded, output:", output);
                     const restartPackages = JSON.parse(output);
                     // Filter out duplicates
-                    restartPackages.reboot = [...new Set(shortenCockpitWsInstance(restartPackages.reboot))];
-                    restartPackages.daemons = [...new Set(shortenCockpitWsInstance(restartPackages.daemons))];
-                    restartPackages.manual = [...new Set(shortenCockpitWsInstance(restartPackages.manual))];
+                    restartPackages.reboot = deduplicate(shortenCockpitWsInstance(restartPackages.reboot));
+                    restartPackages.daemons = deduplicate(shortenCockpitWsInstance(restartPackages.daemons));
+                    restartPackages.manual = deduplicate(shortenCockpitWsInstance(restartPackages.manual));
                     debug("tracer parsed restartPackages:", JSON.stringify(restartPackages));
                     this.setState({ checkRestartAvailable: true, checkRestartRunning: false, restartPackages });
                 })
@@ -1110,6 +1088,7 @@ class OsUpdates extends React.Component {
         const restartPackages = { reboot: [], daemons: [], manual: [] };
 
         // needs-restarting has no machine-readable API: https://issues.redhat.com/browse/RHEL-56139
+        // dnf5 needs-restarting also has no machine-readable API: https://github.com/rpm-software-management/dnf5/issues/2341
         // --exclude-services was added much later, so check that first
         return cockpit.spawn(["dnf", "needs-restarting", "--exclude-services"], { err: "message", superuser: "require" })
                 .then(outManual => {
@@ -1184,102 +1163,33 @@ class OsUpdates extends React.Component {
         this.setState({ state: "loadError" });
     }
 
-    removeHeading(text) {
-        // on Debian the update_text starts with "== version ==" which is
-        // redundant; we don't want Markdown headings in the table
-        if (text)
-            return text.trim().replace(/^== .* ==\n/, "")
-                    .trim();
-        return text;
-    }
-
-    loadUpdateDetails(pkg_ids) {
-        const limit = 500; // Load iteratively to avoid exceeding cockpit-ws frame size
-        PK.cancellableTransaction("GetUpdateDetail", [pkg_ids.slice(0, limit)], null, {
-            UpdateDetail: (packageId, updates, obsoletes, vendor_urls, bug_urls, cve_urls, restart,
-                update_text, changelog /* state, issued, updated */) => {
-                const u = this.state.updates[packageId];
-                if (!u) {
-                    console.warn("Mismatching update:", packageId);
-                    return;
-                }
-
-                u.vendor_urls = vendor_urls;
-                u.description = this.removeHeading(update_text) || changelog;
-                if (update_text)
-                    u.markdown = true;
-                u.bug_urls = deduplicate(bug_urls);
-                // many backends don't support proper severities; parse CVEs from description as a fallback
-                u.cve_urls = deduplicate(cve_urls && cve_urls.length > 0 ? cve_urls : parseCVEs(u.description));
-                if (u.cve_urls && u.cve_urls.length > 0)
-                    u.severity = PK.Enum.INFO_SECURITY;
-                u.vendor_urls = vendor_urls || [];
-                // u.restart = restart; // broken (always "1") at least in Fedora
-
-                this.setState(prevState => ({ updates: prevState.updates }));
-            },
-        })
-                .then(() => {
-                    if (pkg_ids.length <= limit)
-                        this.setState({ state: "available" });
-                    else
-                        this.loadUpdateDetails(pkg_ids.slice(limit));
-                })
-                .catch(ex => {
-                    console.warn("GetUpdateDetail failed:", JSON.stringify(ex));
-                    // still show available updates, with reduced detail
-                    this.setState({ state: "available" });
-                });
-    }
-
     loadUpdates() {
-        const updates = {};
         let cockpitUpdate = false;
 
         this.setState({ state: "loading" });
 
         // check if there is an available version of coreutils; this is a heuristics for unregistered RHEL
         // systems to see if they need a subscription to get "proper" OS updates
-        let have_coreutils = false;
-        PK.cancellableTransaction(
-            "Resolve",
-            [PK.Enum.FILTER_ARCH | PK.Enum.FILTER_NEWEST | PK.Enum.FILTER_NOT_INSTALLED, ["coreutils"]],
-            null,
-            {
-                Package: (info, package_id) => { have_coreutils = true }
-            })
-                .then(() => this.setState({ haveOsRepo: have_coreutils }),
+        PK.is_available(["coreutils"], null)
+                .then((have_coreutils) => this.setState({ haveOsRepo: have_coreutils }),
                       ex => console.warn("Resolving coreutils failed:", JSON.stringify(ex)))
-                .then(() => PK.cancellableTransaction(
-                    "GetUpdates", [0],
-                    data => this.setState({ state: data.waiting ? "locked" : "loading" }),
-                    {
-                        Package: (info, packageId, _summary) => {
-                            const id_fields = packageId.split(";");
-                            packageSummaries[id_fields[0]] = _summary;
-                            // HACK: dnf backend yields wrong severity with PK < 1.2.4 (https://github.com/PackageKit/PackageKit/issues/268)
-                            if (info < PK.Enum.INFO_LOW || info > PK.Enum.INFO_SECURITY)
-                                info = PK.Enum.INFO_NORMAL;
-                            updates[packageId] = { name: id_fields[0], version: id_fields[1], severity: info, arch: id_fields[2] };
-                            if (id_fields[0] == "cockpit-ws")
+                .then(() => PK.get_updates(true, null).then(updates => {
+                    debug("GetUpdates result:", updates);
+                    if (updates.length) {
+                        for (const update of updates) {
+                            if (update.name === 'cockpit-ws') {
                                 cockpitUpdate = true;
                             // Arch Linux has no cockpit-ws package
-                            if (id_fields[0] == "cockpit" && this.state.backend === "alpm")
+                            } else if (update.name === 'cockpit' && this.state.backend === "alpm") {
                                 cockpitUpdate = true;
-                        },
-                    }))
-                .then(() => {
-                    // get the details for all packages
-                    const pkg_ids = Object.keys(updates);
-                    if (pkg_ids.length) {
-                        this.setState({ updates, cockpitUpdate }, () => {
-                            this.loadUpdateDetails(pkg_ids);
-                        });
+                            }
+                        }
+                        this.setState({ updates, cockpitUpdate, state: "available" });
                     } else {
-                        this.setState({ updates: {}, state: "uptodate" });
+                        this.setState({ updates: [], state: "uptodate" });
                     }
                     this.loadHistory();
-                })
+                }))
                 .catch(this.handleLoadError);
     }
 
@@ -1294,7 +1204,12 @@ class OsUpdates extends React.Component {
                     // data looks like:
                     // downloading\tbash-completion;1:2.6-1.fc26;noarch;updates-testing
                     // updating\tbash-completion;1:2.6-1.fc26;noarch;updates-testing
-                const pkgs = { _time: Date.parse(timeSpec) };
+                const _time = Date.parse(timeSpec);
+                if (isNaN(_time)) {
+                    debug(`Transaction has an invalid timespec=${timeSpec}`);
+                    return;
+                }
+                const pkgs = { _time };
                 let empty = true;
                 data.split("\n").forEach(line => {
                     const fields = line.trim().split("\t");
@@ -1318,7 +1233,7 @@ class OsUpdates extends React.Component {
     }
 
     initialLoadOrRefresh() {
-        PK.watchRedHatSubscription(registered => this.setState({ unregistered: !registered }));
+        watchRedHatSubscription(registered => this.setState({ unregistered: !registered }));
 
         cockpit.addEventListener("visibilitychange", () => {
             if (!cockpit.hidden)
@@ -1398,18 +1313,18 @@ class OsUpdates extends React.Component {
     }
 
     applyUpdates(type) {
-        let ids = Object.keys(this.state.updates);
+        let updates = [...this.state.updates];
         if (type === UPDATES.SECURITY)
-            ids = ids.filter(id => this.state.updates[id].severity === PK.Enum.INFO_SECURITY);
+            updates = updates.filter(update => update.severity === Severity.CRITICAL);
         if (type === UPDATES.KPATCHES) {
-            ids = ids.filter(id => isKpatchPackage(this.state.updates[id].name));
+            updates = updates.filter(update => isKpatchPackage(update.name));
         }
 
         PK.transaction()
                 .then(transactionPath => {
                     this.watchUpdates(transactionPath)
                             .then(() => {
-                                PK.call(transactionPath, PK.transactionInterface, "UpdatePackages", [0, ids])
+                                PK.update_packages(updates, null, transactionPath)
                                         .catch(ex => {
                                             // We get more useful error messages through ErrorCode or "PackageKit has crashed", so only
                                             // show this if we don't have anything else
@@ -1446,7 +1361,6 @@ class OsUpdates extends React.Component {
             return <EmptyStatePanel
                 title={_("This system is not registered")}
                 headingLevel="h5"
-                titleSize="4xl"
                 paragraph={ _("To get software updates, this system needs to be registered with Red Hat, either using the Red Hat Customer Portal or a local subscription server.") }
                 icon={ExclamationCircleIcon}
                 action={ _("Register…") }
@@ -1472,13 +1386,12 @@ class OsUpdates extends React.Component {
             else
                 return <EmptyStatePanel loading title={ _("Checking software status")}
                                         headingLevel="h5"
-                                        titleSize="4xl"
                                         paragraph={STATE_HEADINGS[this.state.state]}
                 />;
 
         case "available":
         {
-            const num_updates = Object.keys(this.state.updates).length;
+            const num_updates = this.state.updates.length;
             const num_security_updates = count_security_updates(this.state.updates);
             const num_kpatches = count_kpatch_updates(this.state.updates);
             const highest_severity = find_highest_severity(this.state.updates);
@@ -1505,11 +1418,11 @@ class OsUpdates extends React.Component {
             }
 
             let text;
-            if (highest_severity == PK.Enum.INFO_SECURITY)
+            if (highest_severity == Severity.CRITICAL)
                 text = _("Security updates available");
-            else if (highest_severity >= PK.Enum.INFO_NORMAL)
+            else if (highest_severity >= Severity.IMPORTANT)
                 text = _("Bug fix updates available");
-            else if (highest_severity >= PK.Enum.INFO_LOW)
+            else if (highest_severity >= Severity.LOW)
                 text = _("Enhancement updates available");
             else
                 text = _("Updates available");
@@ -1645,7 +1558,6 @@ class OsUpdates extends React.Component {
             page_status.set_own(null);
             return <EmptyStatePanel loading title={ _("Restarting") }
                                     headingLevel="h5"
-                                    titleSize="4xl"
                                     paragraph={ _("Your server will close the connection soon. You can reconnect after it has restarted.") }
             />;
 
@@ -1687,7 +1599,7 @@ class OsUpdates extends React.Component {
 
     handleRefresh() {
         this.setState({ state: "refreshing", loadPercent: null });
-        PK.cancellableTransaction("RefreshCache", [true], data => this.setState({ loadPercent: data.percentage }))
+        PK.refresh(true, data => this.setState({ loadPercent: data.percentage }))
                 .then(() => {
                     if (this._mounted === false)
                         return;
@@ -1705,7 +1617,7 @@ class OsUpdates extends React.Component {
 
         return (
             <WithDialogs>
-                <Page className='no-masthead-sidebar'>
+                <Page className="pf-m-no-sidebar">
                     {content}
                 </Page>
             </WithDialogs>
