@@ -14,13 +14,14 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with Cockpit; If not, see <http://www.gnu.org/licenses/>.
+ * along with Cockpit; If not, see <https://www.gnu.org/licenses/>.
  */
 
 import cockpit from "cockpit";
 import React from 'react';
 
 import { Alert } from "@patternfly/react-core/dist/esm/components/Alert/index.js";
+import { PageSection } from "@patternfly/react-core/dist/esm/components/Page/index.js";
 
 import { LockIcon } from '@patternfly/react-icons';
 
@@ -29,15 +30,20 @@ import { superuser } from "superuser.js";
 
 const _ = cockpit.gettext;
 
-export class SuperuserAlert extends React.Component {
-    render () {
-        if (superuser.allowed)
-            return null;
+export const SuperuserAlert = () => {
+    if (superuser.allowed || !superuser.configured)
+        return null;
 
-        return <Alert className="ct-limited-access-alert"
-                      variant="warning" isInline
-                      customIcon={<LockIcon />}
-                      actionClose={<SuperuserButton />}
-        title={_("Web console is running in limited access mode.")} />;
-    }
-}
+    // @Venefilyn: We have a PageSection here to get padding when rendered
+    // and avoid that padding when Alert is not rendered. If we start using this
+    // in another place we need to make it work in the aforementioned scenarios.
+    return (
+        <PageSection>
+            <Alert className="ct-limited-access-alert"
+                variant="warning" isInline
+                customIcon={<LockIcon />}
+                actionClose={<SuperuserButton />}
+                title={_("Web console is running in limited access mode.")} />
+        </PageSection>
+    );
+};
