@@ -14,21 +14,20 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with Cockpit; If not, see <http://www.gnu.org/licenses/>.
+ * along with Cockpit; If not, see <https://www.gnu.org/licenses/>.
  */
 
-import React, { useState, useEffect, useRef } from 'react';
+import React, { useState, useEffect, useRef, useMemo } from 'react';
 
 import { Button } from "@patternfly/react-core/dist/esm/components/Button/index.js";
 import { Checkbox } from "@patternfly/react-core/dist/esm/components/Checkbox/index.js";
 import { Card, CardBody, CardHeader, CardTitle } from '@patternfly/react-core/dist/esm/components/Card/index.js';
-import { EmptyState, EmptyStateActions, EmptyStateFooter, EmptyStateHeader, EmptyStateIcon, EmptyStateVariant } from "@patternfly/react-core/dist/esm/components/EmptyState/index.js";
+import { EmptyState, EmptyStateActions, EmptyStateFooter, EmptyStateVariant } from "@patternfly/react-core/dist/esm/components/EmptyState/index.js";
 import { Flex, FlexItem } from "@patternfly/react-core/dist/esm/layouts/Flex/index.js";
 import { HelperText, HelperTextItem } from "@patternfly/react-core/dist/esm/components/HelperText/index.js";
 import { Label, LabelGroup } from "@patternfly/react-core/dist/esm/components/Label/index.js";
 import { Page, PageBreadcrumb, PageSection } from "@patternfly/react-core/dist/esm/components/Page/index.js";
 import { Gallery } from "@patternfly/react-core/dist/esm/layouts/Gallery/index.js";
-import { Select, SelectOption } from "@patternfly/react-core/dist/esm/deprecated/components/Select/index.js";
 import { Breadcrumb, BreadcrumbItem } from "@patternfly/react-core/dist/esm/components/Breadcrumb/index.js";
 import { Form, FormGroup } from "@patternfly/react-core/dist/esm/components/Form/index.js";
 import { TextInput } from "@patternfly/react-core/dist/esm/components/TextInput/index.js";
@@ -38,8 +37,9 @@ import { ExclamationCircleIcon, HelpIcon, UndoIcon } from '@patternfly/react-ico
 
 import cockpit from 'cockpit';
 import { superuser } from "superuser";
-import * as timeformat from "timeformat.js";
+import * as timeformat from "timeformat";
 import { apply_modal_dialog } from "cockpit-components-dialog.jsx";
+import { MultiTypeaheadSelect } from "cockpit-components-multi-typeahead-select";
 
 import { show_unexpected_error } from "./dialog-utils.js";
 import { delete_account_dialog } from "./delete-account-dialog.js";
@@ -96,46 +96,62 @@ function get_expire(name) {
             .then(parse_expire);
 }
 
-export function AccountDetails({ accounts, groups, current_user, user, shells }) {
+export function AccountNotFound() {
+    return (
+        <EmptyState headingLevel="h1" icon={ExclamationCircleIcon} titleText={_("Account not available or cannot be edited.")} variant={EmptyStateVariant.sm} id="account-failure">
+            <EmptyStateFooter>
+                <EmptyStateActions>
+                    <Breadcrumb>
+                        <BreadcrumbItem to="#/">{_("Back to accounts")}</BreadcrumbItem>
+                    </Breadcrumb>
+                </EmptyStateActions>
+            </EmptyStateFooter>
+        </EmptyState>
+    );
+}
+
+export function AccountDetails({ account, groups, isLoading, current_user, shells }) {
     const [expiration, setExpiration] = useState(null);
+
+    const user = useMemo(() => account.name, [account.name]);
+    const [isLocked, setIsLocked] = useState(account.isLocked);
+
+    const [editedRealName, setEditedRealName] = useState(null);
+    const [comittingRealName, setCommittingRealName] = useState(false);
+    const [disableLockedEdit, setDisableLockedEdit] = useState(false);
+
     useEffect(() => {
-        get_expire(user).then(setExpiration);
+        get_expire(account.name).then(setExpiration);
+    }, [account]);
 
-        // Watch `/var/run/utmp` to register when user logs in or out
-        const handle = cockpit.file("/var/run/utmp", { superuser: "try", binary: true });
-        handle.watch(() => {
-            get_expire(user).then(setExpiration);
-        });
-        return handle.close;
-    }, [user, accounts]);
+    // Only update isLocked field if account changes value
+    useEffect(() => {
+        setIsLocked(account.isLocked);
+    }, [account.isLocked]);
 
-    const [edited_real_name, set_edited_real_name] = useState(null);
-    const [committing_real_name, set_committing_real_name] = useState(false);
-
-    const [edited_locked, set_edited_locked] = useState(null);
-
-    function change_real_name() {
-        if (!edited_real_name)
+    function changeRealName() {
+        if (editedRealName === null || editedRealName === undefined)
             return;
 
-        set_committing_real_name(true);
+        setCommittingRealName(true);
 
         // TODO: unwanted chars check
-        cockpit.spawn(["/usr/sbin/usermod", user, "--comment", edited_real_name],
+        cockpit.spawn(["/usr/sbin/usermod", user, "--comment", editedRealName],
                       { superuser: "try", err: "message" })
                 .then(() => {
-                    set_edited_real_name(null);
-                    set_committing_real_name(false);
+                    setEditedRealName(null);
+                    setCommittingRealName(false);
                 })
                 .catch(error => {
-                    set_edited_real_name(null);
-                    set_committing_real_name(false);
+                    setEditedRealName(null);
+                    setCommittingRealName(false);
                     show_unexpected_error(error);
                 });
     }
 
     function change_locked(value, dont_retry_if_stuck) {
-        set_edited_locked(value);
+        setIsLocked(value);
+        setDisableLockedEdit(true);
 
         cockpit.spawn(["/usr/sbin/usermod", user, value ? "--lock" : "--unlock"],
                       { superuser: "require", err: "message" })
@@ -151,13 +167,15 @@ export function AccountDetails({ accounts, groups, current_user, user, shells })
                                     console.log("Account locked state doesn't match desired value, trying again.");
                                     // only retry once to avoid uncontrolled recursion
                                     change_locked(value, true);
-                                } else
-                                    set_edited_locked(null);
+                                }
                             });
                 })
                 .catch(error => {
-                    set_edited_locked(null);
                     show_unexpected_error(error);
+                    setIsLocked(!value);
+                })
+                .finally(() => {
+                    setDisableLockedEdit(false);
                 });
     }
 
@@ -170,32 +188,14 @@ export function AccountDetails({ accounts, groups, current_user, user, shells })
                 .catch(show_unexpected_error);
     }
 
-    if (!accounts.length) {
+    if (isLoading) {
         return (
-            <EmptyState variant={EmptyStateVariant.sm}>
-                <EmptyStateHeader titleText={_("Loading...")} headingLevel="h1" />
+            <EmptyState headingLevel="h1" titleText={_("Loading...")} variant={EmptyStateVariant.sm}>
                 <EmptyStateFooter><Spinner size="xl" /></EmptyStateFooter>
             </EmptyState>
         );
     }
 
-    const account = accounts.find(acc => acc.name == user);
-
-    if (!account) {
-        return (
-            <EmptyState variant={EmptyStateVariant.sm} id="account-failure">
-                <EmptyStateHeader titleText={<>{_("Account not available or cannot be edited.")}</>} icon={<EmptyStateIcon icon={ExclamationCircleIcon} />} headingLevel="h1" />
-                <EmptyStateFooter>
-                    <EmptyStateActions>
-                        <Breadcrumb>
-                            <BreadcrumbItem to="#/">{_("Back to accounts")}</BreadcrumbItem>
-                        </Breadcrumb>
-                    </EmptyStateActions>
-                </EmptyStateFooter>
-            </EmptyState>
-        );
-    }
-
     if (!expiration)
         return null;
 
@@ -218,11 +218,11 @@ export function AccountDetails({ accounts, groups, current_user, user, shells })
     const actions = superuser.allowed && (
         <>
             <Button variant="secondary" onClick={() => logout_account()} id="account-logout"
-              isDisabled={!account.loggedIn || account.uid == 0}>
+              isDisabled={!account.loggedIn || account.uid == 0 || user === current_user}>
                 {_("Terminate session")}
             </Button>
             { "\n" }
-            <Button isDisabled={account.uid == 0} variant="danger" id="account-delete"
+            <Button isDisabled={account.uid == 0 || user === current_user} variant="danger" id="account-delete"
                   onClick={() => delete_account_dialog(account)}>
                 {_("Delete")}
             </Button>
@@ -230,16 +230,16 @@ export function AccountDetails({ accounts, groups, current_user, user, shells })
     );
 
     return (
-        <Page id="account">
-            <PageBreadcrumb stickyOnBreakpoint={{ default: "top" }}>
+        <Page id="account" className="pf-m-no-sidebar">
+            <PageBreadcrumb hasBodyWrapper={false} stickyOnBreakpoint={{ default: "top" }}>
                 <Breadcrumb>
                     <BreadcrumbItem to="#/">{_("Accounts")}</BreadcrumbItem>
                     <BreadcrumbItem isActive>{title_name}</BreadcrumbItem>
                 </Breadcrumb>
             </PageBreadcrumb>
-            <PageSection>
+            <PageSection hasBodyWrapper={false}>
                 <Gallery hasGutter>
-                    <Card className="account-details" id="account-details">
+                    <Card isPlain className="account-details" id="account-details">
                         <CardHeader actions={{ actions }}>
                             <CardTitle id="account-title" component="h2">{title_name}</CardTitle>
                         </CardHeader>
@@ -248,15 +248,15 @@ export function AccountDetails({ accounts, groups, current_user, user, shells })
                                 <FormGroup fieldId="account-real-name" hasNoPaddingTop={!superuser.allowed} label={_("Full name")}>
                                     { superuser.allowed
                                         ? <TextInput id="account-real-name"
-                                                     isDisabled={committing_real_name || account.uid == 0}
-                                                     value={edited_real_name !== null ? edited_real_name : account.gecos}
+                                                     isDisabled={comittingRealName || account.uid == 0}
+                                                     value={editedRealName !== null ? editedRealName : account.gecos}
                                                      onKeyDown={event => {
                                                          if (event.key == "Enter") {
                                                              event.target.blur();
                                                          }
                                                      }}
-                                                     onChange={(_event, value) => set_edited_real_name(value)}
-                                                     onBlur={() => change_real_name()} />
+                                                     onChange={(_event, value) => setEditedRealName(value)}
+                                                     onBlur={() => changeRealName()} />
                                         : <output id="account-real-name">{account.gecos}</output>}
                                 </FormGroup>
                                 <FormGroup fieldId="account-user-name" hasNoPaddingTop label={_("User name")}>
@@ -270,8 +270,9 @@ export function AccountDetails({ accounts, groups, current_user, user, shells })
                                     <Flex spaceItems={{ default: 'spaceItemsSm' }} alignItems={{ default: 'alignItemsCenter' }}>
                                         <FlexItem spacer={{ default: 'spacerNone' }}>
                                             <Checkbox id="account-locked"
-                                                        isDisabled={!superuser.allowed || edited_locked != null || user == current_user}
-                                                        isChecked={edited_locked != null ? edited_locked : account.isLocked}
+                                                        ouiaSafe={disableLockedEdit}
+                                                        isDisabled={!superuser.allowed || disableLockedEdit || user == current_user || isLocked == null}
+                                                        isChecked={isLocked}
                                                         onChange={(_event, checked) => change_locked(checked)}
                                                         label={_("Disallow interactive password")} />
                                         </FlexItem>
@@ -349,11 +350,11 @@ export function AccountDetails({ accounts, groups, current_user, user, shells })
     );
 }
 
-export const AccountGroupsSelect = ({ name, loggedIn, groups, setError }) => {
-    const [isOpenGroup, setIsOpenGroup] = useState(false);
+export const AccountGroupsSelect = ({ name, loggedIn, groups }) => {
     const [selected, setSelected] = useState();
     const [primaryGroupName, setPrimaryGroupName] = useState();
     const [loading, setLoading] = useState(true);
+    const [modifyingGroup, setModifyingGroup] = useState(false);
     const [history, setHistory] = useState([]);
     const previousValue = useRef(null);
 
@@ -384,10 +385,10 @@ export const AccountGroupsSelect = ({ name, loggedIn, groups, setError }) => {
         if (!isUndo)
             setHistory([...history, { type: 'removed', name: group }]);
 
-        setLoading(true);
+        setModifyingGroup(true);
         return cockpit.spawn(["gpasswd", "-d", name, group], { superuser: "require", err: "message" })
                 .then(() => {
-                    setIsOpenGroup(false);
+                    setModifyingGroup(false);
                 }, show_unexpected_error);
     };
 
@@ -395,38 +396,20 @@ export const AccountGroupsSelect = ({ name, loggedIn, groups, setError }) => {
         if (!isUndo)
             setHistory([...history, { type: 'added', name: group }]);
 
-        setLoading(true);
+        setModifyingGroup(true);
         return cockpit.spawn(["gpasswd", "-a", name, group], { superuser: "require", err: "message" })
                 .then(() => {
-                    setIsOpenGroup(false);
+                    setModifyingGroup(false);
                 }, show_unexpected_error);
     };
 
-    const onSelectGroup = (event, selection) => {
-        if (selected.includes(selection)) {
-            removeGroup(selection);
-        } else {
-            addGroup(selection);
-        }
-    };
-
     const chipGroupComponent = () => {
         return (
             <LabelGroup numLabels={10}>
                 {(selected || []).map((currentLabel, index) => {
-                    const optional = currentLabel !== primaryGroupName && superuser.allowed
-                        ? {
-                            onClose: ev => {
-                                ev.stopPropagation();
-                                removeGroup(currentLabel);
-                            }
-                        }
-                        : {};
-
                     return (
                         <Label key={currentLabel}
-                               color={groups.find(group => group.name === currentLabel).isAdmin ? "gold" : "cyan"}
-                               {...optional}
+                               color={groups.find(group => group.name === currentLabel).isAdmin ? "yellow" : "blue"}
                         >
                             {currentLabel}
                         </Label>
@@ -444,27 +427,24 @@ export const AccountGroupsSelect = ({ name, loggedIn, groups, setError }) => {
             validated={history.length > 0 ? "warning" : "default"}
         >
             {superuser.allowed
-                ? <Select
-                   chipGroupComponent={chipGroupComponent()}
-                   isDisabled={!superuser.allowed || loading}
-                   isOpen={isOpenGroup}
-                   onSelect={onSelectGroup}
-                   onToggle={(_, isOpen) => setIsOpenGroup(isOpen)}
-                   selections={selected}
-                   toggleId="account-groups"
-                   variant="typeaheadmulti"
-                >
-                    {groups.map((option, index) => (
-                        <SelectOption
-                            isDisabled={option.name == primaryGroupName}
-                            key={index}
-                            value={option.name}
-                        />
-                    ))}
-                </Select>
+                ? <MultiTypeaheadSelect
+                      isScrollable
+                      isDisabled={loading || modifyingGroup}
+                      onAdd={val => addGroup(val)}
+                      onRemove={val => removeGroup(val)}
+                      options={groups.map((option, index) => {
+                          return {
+                              value: option.name,
+                              content: option.name,
+                              color: option.isAdmin ? "yellow" : "blue",
+                              isDisabled: option.name == primaryGroupName,
+                          };
+                      })}
+                      selected={selected || []}
+                      toggleProps={{ id: "account-groups" }} />
                 : chipGroupComponent()}
             {(history.length > 0)
-                ? <HelperText className="pf-v5-c-form__helper-text">
+                ? <HelperText className="pf-v6-c-form__helper-text">
                     <Flex>
                         {loggedIn && <HelperTextItem id="account-groups-helper" variant="warning">{_("The user must log out and log back in for the new configuration to take effect.")}</HelperTextItem>}
                         {history.length > 0 && <Button variant="link" id="group-undo-btn" isInline icon={<UndoIcon />} onClick={undoGroupChanges}>{_("Undo")}</Button>}
